/****************************************************************************************
 * DMX through UDP / Lanbox connection
 * (see http://vvvv.org/sites/default/files/user-files/LCplus%20Reference%20v2.0a1.pdf)
 * 
 * No need to change this!
 ****************************************************************************************/

import hypermedia.net.*;

//=================================
//DMX stuff for the main program

DMXUDPsender dmxUdpSender;

void setupDMX(String ip) {
    dmxUdpSender = new DMXUDPsender(ip);
}

void stopDMX() {
    dmxUdpSender.free();
    dmxUdpSender = null;
}



//=================================
//class for sending DMX through UDP

public class DMXUDPsender
{
    UDP udp;
    String pLanboxIP;
    int pPort;

    DMXUDPsender(String aip) {
        pLanboxIP = aip;
        // supposes that channels are in the order RGB
        pPort = 4777;
        udp = new UDP(this);
    }

    void  free() {
        if (udp != null) {
            udp.close();
            udp = null;
        }
    }

    // value is 0..255
    void resend(int [] data) {

        /**
         * header: 4 bytes
         * message
         * message header: 6 bytes
         * data: nb spots * 3 bytes (3 channels or colours)
         * padding: 1 byte
         */
        int dataLen = data.length; //length of the colour information = number of spots * 3 (RGB)
        int msgLen = 6 + dataLen;
        int bufferLen = 4 + msgLen + 1;

        byte [] buffer = new byte[bufferLen];

        // =================================
        // HEADER
        // ==============
        /*0*/        buffer[0]   = byte(0xC0);
        /*1*/        buffer[1] = byte(0xB7);
        /*2*/        buffer[2] = byte(0x0);
        /*3*/        buffer[3] = byte(0x0);

        // =================================
        // MESSAGE
        // ==============
        // MESSAGE HEADER
        /*4*/        buffer[4] = byte(0xCA); // "8-bit message type, always 0xCA"
        /*5*/        buffer[5] = byte(254); // "The buffer id must be 254 to write into the mixer buffer"

        // 16-bit message length (of entire message, except padding)
        /*6*/        buffer[6] = (byte)(HighByte(msgLen) ); // HighByte length HEXA
        /*7*/        buffer[7] = (byte)( LowByte(msgLen) ); // LowByte lenght HEXA    

        // 16-bit channel offset: first channel at 10 = A in hexadecimal = 0xA
        /*8*/        buffer[8] = (byte)(HighByte(0xA)); // HighByte first channel
        /*9*/        buffer[9] = (byte)( LowByte(0xA)); // LowByte first channel

        // MESSAGE CONTENT
        // (adding data/colours to the buffer)
        for (int i=10; i<10+dataLen; i++) {
            buffer[i]=byte(data[i-10]);
        }

        // =================================
        // PADDING
        // ==============
        if ((10+dataLen)%2!=0)
            buffer[10+dataLen]= byte(0xFF);
        else {
            buffer=shorten(buffer);//?????
        }

        // sending the buffer through UDP
        udp.send(buffer, pLanboxIP, pPort);
    }
}


//bytes manipulation (for 16bits formats)
int HighByte(int a) {
    return (a >> 8);
}

int LowByte(int a) {
    return (a & 0xFF);
}

