import processing.core.*; 
import processing.xml.*; 

import colorLib.calculation.*; 
import colorLib.*; 
import colorLib.webServices.*; 
import oscP5.*; 
import netP5.*; 
import hypermedia.net.*; 
import java.util.Collections; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class SmileTransporter extends PApplet {








OscP5 osc;

SmileFacade facade;

Palette p;
PImage colorImage;
int MAX_SWATCHES;
Palette smileColors;

String dmxIP = "192.168.1.77";
int nbSpots = 18;

public void setup()
{
    size( 1050, 600 );

    colorImage = loadImage( "smilecolors.png" );
    p = new Palette( this, colorImage );
    MAX_SWATCHES =  p.totalSwatches();

    facade = new SmileFacade( this, nbSpots );
        
    osc = new OscP5( this, 12345 );
    
    smileColors = new Palette( this );
    smileColors.addColor( color( 255, 37, 29 ) );
    smileColors.addColor( color( 217, 28, 88 ) );
    smileColors.addColor( color( 255, 0, 0 ) );
    
    frameRate( 25 );
}

public void draw()
{
    background( 0 );
    facade.update();
    facade.scrDraw();
    facade.dmxDraw();
}

public void oscEvent( OscMessage msg )
{
    
    if ( msg.checkAddrPattern("/smile") == true ) {
        if ( msg.checkTypetag("i") ) {
            int smiled = msg.get(0).intValue();
            if ( smiled == 1 ) {
                facade.smiled = true;
            } else {
                facade.smiled = false;
            }
            facade.modeChanged();
        } 
    }
}
/****************************************************************************************
 * DMX through UDP / Lanbox connection
 * (see http://vvvv.org/sites/default/files/user-files/LCplus%20Reference%20v2.0a1.pdf)
 * 
 * No need to change this!
 ****************************************************************************************/



//=================================
//DMX stuff for the main program

DMXUDPsender dmxUdpSender;

public void setupDMX(String ip) {
    dmxUdpSender = new DMXUDPsender(ip);
}

public void stopDMX() {
    dmxUdpSender.free();
    dmxUdpSender = null;
}



//=================================
//class for sending DMX through UDP

public class DMXUDPsender
{
    UDP udp;
    String pLanboxIP;
    int pPort;

    DMXUDPsender(String aip) {
        pLanboxIP = aip;
        // supposes that channels are in the order RGB
        pPort = 4777;
        udp = new UDP(this);
    }

    public void  free() {
        if (udp != null) {
            udp.close();
            udp = null;
        }
    }

    // value is 0..255
    public void resend(int [] data) {

        /**
         * header: 4 bytes
         * message
         * message header: 6 bytes
         * data: nb spots * 3 bytes (3 channels or colours)
         * padding: 1 byte
         */
        int dataLen = data.length; //length of the colour information = number of spots * 3 (RGB)
        int msgLen = 6 + dataLen;
        int bufferLen = 4 + msgLen + 1;

        byte [] buffer = new byte[bufferLen];

        // =================================
        // HEADER
        // ==============
        /*0*/        buffer[0]   = PApplet.parseByte(0xC0);
        /*1*/        buffer[1] = PApplet.parseByte(0xB7);
        /*2*/        buffer[2] = PApplet.parseByte(0x0);
        /*3*/        buffer[3] = PApplet.parseByte(0x0);

        // =================================
        // MESSAGE
        // ==============
        // MESSAGE HEADER
        /*4*/        buffer[4] = PApplet.parseByte(0xCA); // "8-bit message type, always 0xCA"
        /*5*/        buffer[5] = PApplet.parseByte(254); // "The buffer id must be 254 to write into the mixer buffer"

        // 16-bit message length (of entire message, except padding)
        /*6*/        buffer[6] = (byte)(HighByte(msgLen) ); // HighByte length HEXA
        /*7*/        buffer[7] = (byte)( LowByte(msgLen) ); // LowByte lenght HEXA    

        // 16-bit channel offset: first channel at 10 = A in hexadecimal = 0xA
        /*8*/        buffer[8] = (byte)(HighByte(0xA)); // HighByte first channel
        /*9*/        buffer[9] = (byte)( LowByte(0xA)); // LowByte first channel

        // MESSAGE CONTENT
        // (adding data/colours to the buffer)
        for (int i=10; i<10+dataLen; i++) {
            buffer[i]=PApplet.parseByte(data[i-10]);
        }

        // =================================
        // PADDING
        // ==============
        if ((10+dataLen)%2!=0)
            buffer[10+dataLen]= PApplet.parseByte(0xFF);
        else {
            buffer=shorten(buffer);//?????
        }

        // sending the buffer through UDP
        udp.send(buffer, pLanboxIP, pPort);
    }
}


//bytes manipulation (for 16bits formats)
public int HighByte(int a) {
    return (a >> 8);
}

public int LowByte(int a) {
    return (a & 0xFF);
}

/****************************************************************************************
 * Default facade-pixels manager, to be extended by your own class
 * 
 * No need to change this!
 ****************************************************************************************/


class Facade
{

    int[] grid;//array of colours, one per spot/window
    int nbSpots;


    Facade(int n)
    {
        nbSpots = n;
        grid = new int[nbSpots];//one per spot/window

        //setting up DMX stuff
        setupDMX(dmxIP);
    }

    //default updater : random
    public void update()
    {
        for (int i = grid.length-1;i>=0;i--) {
            grid[i]= color(random(255), random(255), random(255));
        }
    }

    //draw the grid on screen
    public void scrDraw()
    {
        colorMode(HSB);
        for (int i = grid.length-1;i>=0;i--) {
            fill(grid[i]);
            rect(20+i*80, 200, 50, 50);
        }
    }

    //send the grid to dmx
    public void dmxDraw()
    {
        int [] buffer = new int[nbSpots*3]; //dmx data to be sent. Size = nb spots * 3 channels/colours
        int j=0;
        int c;
        colorMode(RGB);
        for (int i = 0; i < grid.length; i++) {
            c=grid[i];
            buffer[j]=(int)red(c); 
            j++; 
            buffer[j]=(int)green(c); 
            j++; 
            buffer[j]=(int)blue(c); 
            j++;
        }

        dmxUdpSender.resend(buffer);
        // println(buffer);
    }

    public void click() {
    }
}



class SmileFacade extends Facade
{

    boolean smiled;
    
    int randomColor; // used for patterns
    int randomSmileColor;
    float fadePercent;
    
    int currentColor;
    
    int[] patternColors;
    float patternFadePercent;
    int gradientOffset;
    
    PApplet papplet;
    
    int currentMode;
    int prevMode;
    
    ArrayList<Integer> randomNumbers;
    
    SmileFacade( PApplet _papplet, int n )
    {
        super( n );
        papplet = _papplet;
        
        smiled = false;
        
        fadePercent = 0.0f;
        patternFadePercent = 0.0f;
        
        currentColor = color( 255 );
                
        randomColor = floor( random( MAX_SWATCHES ) );
        randomSmileColor = floor( random( 3 ) );
                
        randomNumbers = new ArrayList<Integer>();
        for ( int i = 0; i < n; i++ ) {
            randomNumbers.add( i );
        }
        
//        println( randomNumbers );
        Collections.shuffle( randomNumbers );
        
        println( randomNumbers );
        patternColors = new int[ n ];
        println( "Patterncolors Length: " + patternColors.length );
        fillPatternColors();
        
        prevMode = 0;
        currentMode = 0;
        
        gradientOffset = 0;
        
    }
    
    public void update()
    {
                
        if ( smiled ) {
                
            patternFadePercent = 0.0f;

            currentColor = lerpColor( smileColors.getColor( randomSmileColor ), color( 0 ), fadePercent );
            fadePercent += 0.005f;

            grid[ 0 ] = currentColor;
            grid[ 5 ] = currentColor;
            grid[ 6 ] = currentColor;
            grid[ 7 ] = currentColor;
            grid[ 10 ] = currentColor;
            grid[ 11 ] = currentColor;
            grid[ 13 ] = currentColor;
            grid[ 14 ] = currentColor;
            grid[ 15 ] = currentColor;
            grid[ 16 ] = currentColor;

            grid[ 1 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 2 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 3 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 4 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 8 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 9 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 12 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
            grid[ 17 ] =  color( 0 ); //lerpColor( currentColor, color( 0 ), 0.75 );
                
        } else {
            // not smiling. Create a pattern and draw it to the screen
            fadePercent = 0.0f;
            patternFadePercent += 0.004f;
            for ( int i = 0; i < grid.length; i++ ) {
                grid[ i ] =  lerpColor( patternColors[i], color( 0 ), patternFadePercent );
            }
        }
        
        if ( fadePercent >= 1 ) {
            fadePercent = 0.0f;
            smiled = false;
            randomColor = floor( random( MAX_SWATCHES ) );
        }
        
        if ( patternFadePercent >= 1 ) {
            patternFadePercent = 0.0f;
            fillPatternColors();
            gradientOffset += 1;
            if ( gradientOffset >= 3 ) {
                gradientOffset = 0;
            }
        }
    }
    
    public void scrDraw()
    {
        int counter = 0;
        
        stroke( 255, 64 );
        
        pushMatrix();
        translate( 75, 75 );
        for ( int i = 0; i < 6; i++ ) {
            fill( grid[ counter ] );
            rect( i * 150, 0, 100, 100 );
            counter++;
        }
        translate( 0, 150 );
        for ( int i = 0; i < 6; i++ ) {
            fill( grid[ counter ] );
            rect( i * 150, 0, 100, 100 );
            counter++;
        }
        translate( 0, 150 );
        for ( int i = 0; i < 6; i++ ) {
            fill( grid[ counter ] );
            rect( i * 150, 0, 100, 100 );
            counter++;
        }
        popMatrix();
    }

    public void fillPatternColors()
    {
        boolean gradientMode = false;
        if ( random( 100 ) < 50 ) {
            gradientMode = true;
        }

        int[] grc = new int[2];
        grc[0] = p.getColor( floor( random( MAX_SWATCHES ) ) );
        grc[1] = p.getColor( floor( random( MAX_SWATCHES ) ) );
        Gradient gr = new Gradient( papplet, grc, 6 );
        
        if ( gradientMode ) {
            for ( int i = 0; i < patternColors.length; i++ ) {
                patternColors[i] = color( 0 );       
            }
            for ( int i = 0; i < 6; i++ ) {
                patternColors[ i + ( gradientOffset * 6 )] = gr.getColor( i );
            }
    
        } else {
            
            Collections.shuffle( randomNumbers );
            
            for ( int i = 0; i < patternColors.length; i++ ) {
                patternColors[i] = color( 0 );       
            }
            
            for ( int i = 0; i < 6; i++ ) {
                int n = randomNumbers.get( i );
                patternColors[ (int)randomNumbers.get(i) ] = gr.getColor( i );
            }
            println("---");
        
        }

    }
    
    public void modeChanged()
    {
        prevMode = currentMode;
        
        boolean changed = false;
        if ( smiled ) {
            currentMode = 1;
        } else {
            currentMode = 0;    
        }
        
        if ( currentMode != prevMode ) {
            changed = true;
        }
        
        if ( changed ) {
            patternFadePercent = 0.0f;
            fillPatternColors();
            fadePercent = 0.0f;
            randomColor = floor( random( MAX_SWATCHES ) );
            randomSmileColor = floor( random( 3 ) );
        }
       
    }

}
    static public void main(String args[]) {
        PApplet.main(new String[] { "--bgcolor=#ffffff", "SmileTransporter" });
    }
}
